#!/bin/sh
#
# Copyright 1995 Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)installsvr3.sh	1.10	95/09/25 SMI"
#
:

trap 'exit 0' 1 2 3 15

INDIR=/usr/lib/installed
OPTDIR=/usr/options
CONTENTS=${INDIR}/CONTENTS
PLUSDIR=/tmp/installed
TMPDIR=/usr/tmp/install$$
CPLOG=/tmp/cplog$$
TERMINFO=/usr/share/lib/terminfo       # compiled terminfo descriptions
SYSV3=${SYSV3:-""}
export SYSV3
setvar -ui SYSV3 "$SYSV3"

TFILES="at386-m AT386-M 386at-m 386AT-M at386 AT386 386at 386AT at386-ul \
AT386-UL 386at-ul 386AT-UL"

FDMESS=`gettext '\nIf the program installation requires more than one floppy disk, be sure \nto insert the disks in the proper order, starting with disk number 1.\nAfter the first floppy disk, instructions will be provided for inserting \nthe remaining floppy disks.\n'`

ERROR2=`gettext 'An error was encountered while reading in the floppy disk(s).\nPlease be sure to insert them in the proper order, that the drive door \nis closed, and wait for the notification before removing them.\n'`

ERROR3=`gettext 'If this problem reoccurs at the same floppy disk, the floppy disk may \nbe bad. Please re-insert the first floppy disk of this package and try again.\n'`

Msg1=`gettext 'Please enter the number of floppies in the package followed by ENTER: \c'`
Msg2=`gettext '\007The number of floppies field must be filled in.\007\n'`
Msg3=`gettext '\007The number of floppies, is not a valid number.\007\n'`
Msg4=`gettext '\007The number of floppies must be at least 1.\007\n'`
Msg5=`gettext 'Warning: This software pacakge appears to contain SVR3.2 device driver.\nrelated component.  SVR3.2 device driver is not supported under Solaris.\nInstallation will proceed but the device driver component will not be\nsupported.\n'`

MediaMsg=`gettext '\rPlease enter:\n1 (for 360 KB)\n2 (for 1.2 MB)\n3 (for 720 KB)\n4 (for 1.44 MB)\n\c'`

########
# save-term (No Arg)
# 	save existing terminfo descriptions
#
save_term()
{
    SAVEDIRS=
    for f in $TFILES ; do
	grp=`echo $f | cut -c1`
	[ -d $TERMINFO/$grp ] || continue
	[ -d $TERMINFO/$grp/.isave ] || mkdir $TERMINFO/$grp/.isave
	cp $TERMINFO/$grp/$f $TERMINFO/$grp/.isave >/dev/null 2>&1
	expr "$SAVEDIRS" : ".*$grp" >/dev/null 2>&1  ||
		 SAVEDIRS="$SAVEDIRS $grp"
    done
}

########
# restore_term (No Arg)
#
#	restore the terminfo description previously saved (by save_term).
#
restore_term()
{
    for d in $SAVEDIRS
    do
	cp $TERMINFO/$d/.isave/* $TERMINFO/$d && rm -rf $TERMINFO/$d/.isave
    done
    unset SAVEDIRS
}


########
#
#
#
#
confirm()
{
	rv=0
	gettext 'Strike ENTER when ready\nor <q> follow by <ENTER> to stop '
	read jnk

	[ "$jnk" = "q" ] && rv=1
	return $rv
}


########
# cancel ()
#
#
install_cancel()
{
	gettext 'The Installation is canceled'
	exit 1
}

IdSpaceMsg()
{
	TEXT=`gettext 'Not enough space on $FSYS to install the package.\nPlease remove some files on $FSYS and try again.\n'`
	eval echo $TEXT
}

########
# IdSpace (Arg1, Arg2)
#
#	a more brain damage implementation of /etc/conf/bin/idspace program
#	(exist only on true SVR3.2 system).
#
IdSpace()
{
	ROOT="$1"
	USR="$2"
	export ROOT USR

	df -lk | nawk '
	BEGIN {
		"echo $ROOT" | getline RSize
		"echo $USR" | getline USize

		User = RReminder = UReminder = 0

		RSize = RSize * 512 / 1000
		USize = USize * 512 / 1000
	}

	$6 == "/"  || $6 == "/usr" {
		split($5, a, "%")

		if ($6 == "/") {
			RAvail=$4
			RCap=a[1]
		} else {
			UAvail=$4
			UCap=a[1]
			User = 1
		}
	}

	END {
		if (User == 0) {
			RReminderinder = ( RAvail - (Rsize + Usize))
			if (RReminderinder < 0) {
				fs_name("/");
				exit 1
			}
		} else {
			RReminder = (RAvail - RSize)
			UReminder = (UAvail - USize)

			if (RReminder < 0) {
				fs_name("/");
				exit 1
			} else if (UReminder < 0) {
				fs_name("/usr");
				exit 1
			}
		}
		exit 0
	}

	function fs_name(fs) {
		printf "%s\n", fs
	}
	'
}


############################################################################
# MAIN()
#
############################################################################

DRIVE=$1
[ -z "$DRIVE" ] && gettext 'You must specify the media to install from.\n' && exit 1

########
# Check if root is performing the operation
########
id | grep "(root)" > /dev/null
if [ "$?" != "0" ] ; then
	gettext 'You must be root or super-user to install software.\n'
	exit 1
fi

[ -z "$TERM" ] && TERM=AT386-M
[ -z "$TMPDIR" ] && TMPDIR=/usr/tmp/install$$
[ -z "$CONTENTS" ] && CONTENTS=${INDIR}/CONTENTS
[ ! -d "$OPTDIR" ] && mkdir -p "$OPTDIR" && chmod 755 "$OPTDIR"
CLEAR=""
FSYS=""

cd / ;  rm -rf ${TMPDIR};  mkdir ${TMPDIR}; cd ${TMPDIR}
[ ! -f ${CONTENTS} ] && \
	rm -rf ${CONTENTS} && mkdir -p `dirname ${CONTENTS}` && >> ${CONTENTS}

echo "$FDMESS"
confirm
[ $? != 0 ] && exit 1

USR=10
ROOT=10

########
# Start the installation.
########
gettext '\n\nInstallation is in progress - do not remove the media\n'

cd $TMPDIR
cpio -iBcduH odc -I${DRIVE} -M 'You may remove this floppy disk.\nTo QUIT - strike <q> followed by <ENTER>\nTo continue - insert floppy disk number %d and strike the <ENTER> key.' 2>&1 | tee ${CPLOG}

err=$?

grep "Impossible header type." ${CPLOG} >/dev/null 2>&1
if [ "$?" = "0" ] ; then
	TEXT=`gettext 'The floppy disk you inserted is either not the correct floppy disk, or you inserted it in the wrong order. ${ERROR3}'`
	eval echo $TEXT
	gettext 'Installation aborted.' && exit 1
fi

grep "annot create" ${CPLOG} >/dev/null 2>&1
if [ "$?" = "0" ] ; then
	TEXT=`gettext 'Your hard disk is out of space.  Please remove some files and try again.'`
	eval echo $TEXT
	gettext 'Installation aborted.' && exit 1

fi

if [ "$err" = "4" ] ; then
	TEXT=`gettext 'You have canceled the Installation.  If you wish \nto try it again please retry this operation.'`
	eval echo $TEXT
	gettext 'The Installation is canceled.' && exit 1

elif [ "$err" != "0" ] ; then
	echo ${ERROR2} ${ERROR3}
	[ $? != "0" ] && install_cancel && exit 1

fi

if [ ! -f ./Install -o ! -f ./Remove -o ! -f ./Name -o ! -f ./Size ] ; then
	TEXT=`gettext 'The software package is missing the necessary installation programs.\nPlease check to make sure you have the right floppy disk(s).'`
	eval echo $TEXT
	confirm
	[ "$?" != "0" ] && install_cancel && exit 1
fi

ROOT=`grep ROOT Size | cut -d= -f2`
USR=`grep USR Size | cut -d= -f2`
FSYS=`IdSpace $ROOT $USR`
[ $? != 0 ] && IdSpaceMsg && install_cancel && exit 1

chmod +x * >/dev/null 2>&1
NAME=`cat -s Name`
########
# Check out if previously installed
########
SNAME=`echo "$NAME" | sed -e 's/[ &()$#-?\\!*;|<>]/./g' -e 's/\[/./g' -e 's/\]/./g'`

########
#  Warn the user if there is a xenix package already installed
#  with the same name.  
#  Use a bogus directory to check if /etc/perms is an empty 
#  directory.  Used to be guaranteed not empty in XENIX.
#  Avoids expansion of '*'.
########
tmp=/tmp/CUSLIST
perm_files=`cd /etc/perms; echo *`
if [ "/att/msoft/isc/*" != "/att/msoft/isc/$perm_files" ] ; then
	for perm in $perm_files ; do
		########
		#  Call fixperm to list the installed packages 
		#  for this set
		########
		fixperm -iv $ignorepkgs /etc/perms/$perm |
		sed "s/^\(.*\)	.*$/s:^#!\1[ 	][ 	]*:	&	:p/" > $tmp.fl
		########
		# Get the package name in field 4 
		########
		sed -n -f $tmp.fl /etc/perms/$perm   | 
				sed "s/^	//"  | 
				cut -f4 >> $tmp.sid
	done

	KEY=`grep "$SNAME\$" $tmp.sid`
	rm $tmp.sid $tmp.fl
	if [ -n "$KEY" ] ; then 
		TEXT=`gettext '\nA XENIX package with the same name, $NAME,\nhas already been installed.  This package may overwrite the XENIX package.\n'`
		eval echo ${TEXT}
		confirm
		[ "$?" != "0" ] && install_cancel && exit 1
	fi

	KEY=`grep " $SNAME\$" $CONTENTS`
	if [ -n "$KEY" ] ; then
		TEXT=`gettext '\nThe $NAME has already been installed.\nThe new installation will now replace the original $NAME files.\n'`
		eval echo ${TEXT}
		confirm
		[ "$?" != "0" ] && install_cancel && exit 1

		cd /usr/options
		for i in * ; do
			grep "^${SNAME}$" $i > /dev/null 2>&1
			[ $? = 0 ] && KEY2=$i
	     	done

		cd ${TMPDIR}
		eval KEY=`grep " $SNAME\$" $CONTENTS | cut -f1 -d' ' `
		if [ "$KEY" ] ; then
			mv ${INDIR}/Remove/${KEY2} /tmp/${KEY}.R
			mv ${INDIR}/Files/${KEY2} /tmp/${KEY}.F > /dev/null 2>&1
			mv /usr/options/${KEY2} /tmp/${KEY}.O
			grep " $SNAME\$" $CONTENTS > /tmp/${KEY}
			grep -v " $SNAME\$" $CONTENTS > /tmp/$$
			mv /tmp/$$ $CONTENTS
		fi
	else
		KEY2=`echo "$NAME" | sed -e 's/ //g' | tr "[A-Z]" "[a-z]" | cut -c1-6`
		if [ -f /usr/options/${KEY2}.name ] ; then
			num=`expr 1`
			while [ -f /usr/options/${KEY2}${num}.name ]
			do
				num=`expr $num + 1`
			done
			KEY2=${KEY2}${num}.name
		else
			KEY2=${KEY2}.name
		fi
	fi
		
	########
	# Work-around for problem with Norton Utilities over-writing
	# console terminfo files.
	########
	TERMSAVE=0
	if echo $NAME | grep -s "Norton Utilities" >/dev/null 
	then
		TERMSAVE=1
		save_term
	fi

	########
	# OK, install it now.
	########
	echo "$CLEAR "
	./Install
	rc=$?
	[ "$TERMSAVE" = "1" ] && restore_term

	########
	# It is the 'Install' script's job to print an error message
	# if the installation indicates the return of non-zero:
	########
	if [ "$rc" != "0" ] ; then
		if [ -n "$KEY" ] ; then
		    mv /tmp/${KEY}.R ${INDIR}/Remove/${KEY2} >/dev/null 2>&1
		    mv /tmp/${KEY}.F ${INDIR}/Files/${KEY2} >/dev/null 2>&1
		    mv /tmp/${KEY}.O /usr/options/${KEY2} >/dev/null 2>&1
		    cat /tmp/${KEY} >> $CONTENTS
		    rm -f /tmp/${KEY} > /dev/null 2>&1
		fi
	else
		if [ -n "$KEY" -a -s /tmp/${KEY}.un ] ; then
			rm -f /tmp/${KEY}.[RFO] /tmp/${KEY}
		fi
		[ ! -d $INDIR ] && mkdir $INDIR && chmod 755 $INDIR
		if [ ! -d ${INDIR}/Files ] ; then
			mkdir ${INDIR}/Files
			chmod 755 ${INDIR}/Files
		fi
		if [ ! -d ${INDIR}/Remove ] ; then
			mkdir ${INDIR}/Remove
			chmod 755 ${INDIR}/Remove
		fi
		echo "$KEY2 1 $NAME" >> $CONTENTS
		mv ./Remove ${INDIR}/Remove/${KEY2}
		mv ./Name /usr/options/${KEY2}
		[ -f Files ] && mv Files ${INDIR}/Files/${KEY2}

		sync; sync
		if [ -f /etc/.new_unix ] ; then
			sync; sync
			rm -f /etc/.new_unix
			echo $Msg5
		else
			TEXT=`gettext 'The installation of the $NAME is now complete.'`
			eval echo $TEXT
		fi
	fi
fi

cd /
rm -rf ${TMPDIR} ${CPLOG} &
freevar SYSV3
exit 0
