#
# Copyright 1995 Sun Microsystems, Inc.
# All rights reserved.
#
#pragma ident	"@(#)pkgadd.sh	1.19	95/12/01 SMI"
#
#UI-cmd#

trap 'exit 0' 1 2 3 15

authorized || exit 0

########
# clean (No Arg)
#
#	function to perform house cleaning at the end of the procedure
#	or when bail out for some other reasons such as failure.
#
clean()
{
	CMD=${THISCMD_ADD}
	CURRWD=${HERE_ADD}
	cleanup ${DRIVE_LST} ${DRIVE_FILE}
	freevar DEV RDEV
}


#############################################################################
# Initialize variables:
#
ROOTDIR=${1}
HERE_ADD=`pwd`
THISCMD_ADD=${CMD}
dev=directory
DRIVE_LST=${UITMPPATH}/drives.${CMD}
DRIVE_FILE=${UITMPPATH}/drivefile.${CMD}

setvar -ui DEV		""
setvar -ui RDEV		""

########
# probe_dev (No Arg)
#
#	probe the system for the configured CDROM and tape devices.
#
probe_dev()
{

	########
	# CDrom
	########
	IDX=0
	wait
	fgrep cdrom $DEVLINKS_FILE | sed -n 's/.*entry:\([^:]*\):.*/\1/p' | 
		uniq | while read line  ; do
			device=`ls -l /dev/dsk/c*p0 |
				grep $line |
				awk '{ print $9 }'`

			echo cdrom${IDX}=$device
			IDX=`expr $IDX + 1`
		done

	########
	# Tape
	########
	IDX=0
	fgrep tape $DEVLINKS_FILE | sed -n 's/.*entry:\([^:]*\):.*/\1/p' | 
		uniq | while read line  ; do
			device=`ls -l /dev/rmt/[0-9] |
				grep $line |
				awk '{ print $9 }'`

			echo tape${IDX}=$device
			IDX=`expr $IDX + 1`
		done 
}


########
# init_list: 
#
#	Initialize pick lists.
#

# The devlinks command should really be run in the function "prob_dev", but
# I'm running it now in the background to increase performance
DEVLINKS_FILE=${UITMPPATH}/devlinks.${CMD}
devlinks -d > $DEVLINKS_FILE 2>&1 &

/bin/ls /dev/diskette? | sed 's/\/dev\///g'  > ${DRIVE_LST}
probe_dev > $DRIVE_FILE
cut -d'=' -f1 $DRIVE_FILE >> ${DRIVE_LST}
UI message lookup %media >> ${DRIVE_LST}

########
# Main()
#
UI file ${CMDdotUI} ${DRIVE_LST}

if chkdepends -q %custom_pkgs ; then
	SelectSVR4="0"			# This is not the automatic choice
else
	# Because custom/iBCS2 is not installed, get rid of all the SVR3 options
	UI batch <<-EOF
		field attrib A 		NODISPLAY 
		field attrib AUTO 	NODISPLAY 
		field attrib T 		NODISPLAY
		field attrib SVR3 	NODISPLAY
		field attrib I 		NODISPLAY
		field attrib IUS 	NODISPLAY
		field attrib S 		NODISPLAY
		field attrib CUSTOM	NODISPLAY
		form field Drives select 
	EOF
	SelectSVR4="1"		# This is the only choice, so select it.
fi
############################################################################
# Step 1. Present all the possible media to the user and Package Types.
#	  If the media selected is floopy and pkgtype is auto then pre-
#	  process the media 1st. else fall through step 2.
############################################################################

UI batch <<EOF
	window select main 
	window select Drives 
	form edit Drives "" "" "$SelectSVR4"
	picklist hide drives_pick
	form hide Drives
	window hide Drives
EOF

eval `UI form get Drives` 	# This sets PkgType and drive variables

########
# cancel. just exit.
########
[ $button = dcancel ] && clean && exit 0

############################################################################
# Step 2. Depending on the media selected, go off and do the installation by
#	  calling the proper type of installation program.
#
#	directory	: currently support SVR4 package installation only.
#			  Installation is done in non-interactive mode.
#
#	diskette?	: Auto package detect or manual select type of 
#			  package (SVR4, IUS, AT&T SVR3, SCO custom)
#			  Installation is done in interactive mode.
#
#	cdrom		: Treat it like directory.
#			 
#	tape		: Unknown frontier.
#
############################################################################

UI infomsg %Getting_contents_i
case "${drive}" in

   diskette?)
	rdev=`UI variable get RDEV`
	dev=`UI variable get DEV`

	case "${PkgType}" in
	   1)
		########
		# SVR4 Package
		########
		run_cmd ${HERE_ADD}/svr4/svr4.sh $drive $dev $ROOTDIR 
		;;
	   2)
		########
		# AT&T SVR3 Package
		########
		run_cmd ${HERE_ADD}/installpkg/installpkg.sh $rdev 
		;;
	   3)
		########
		# IUS Package
		########
		run_cmd ${HERE_ADD}/ius/ius.sh $dev 
		;;
	   4)
		########
		# SCO Package
		########
		run_cmd ${HERE_ADD}/custom/custom.sh $rdev diskette
		;;
	esac
	;;

   cdrom?)
	########
	# Find out if vold is running and set devices appropriately.
	########
	drivetype=`echo $drive | sed 's/.$//'`

	if [ "${drivetype}" = "cdrom" -a -d /vol/dev ] ; then
		volcheck
		dev=/cdrom
		run_cmd ${HERE_ADD}/svr4/svr4.sh $drivetype $dev $ROOTDIR 
		eject $drive
	else
		dev=`grep $drive $DRIVE_FILE | cut -d'=' -f2`
		[ ! -d "/$drive" ] && mkdir "/$drive"
		mount -rf hsfs ${dev} "/$drive" 2>${ERROR_FILE}
		[ $? -ne 0 ] && UI call errmsg ${ERROR_FILE} && clean && exit 0
		dev="/$drive"
		run_cmd ${HERE_ADD}/svr4/svr4.sh $drivetype $dev $ROOTDIR 
		umount "/$drive"
	fi
	;;	


   tape?)
	dev=`grep $drive $DRIVE_FILE | cut -d'=' -f2`
	case "${PkgType}" in
	1)
		########
		# SVR4 Package
		########
		run_cmd ${HERE_ADD}/svr4/svr4.sh $drive $dev $ROOTDIR 
		;;
	4)
		########
		# SCO Package
		########
		run_cmd ${HERE_ADD}/custom/custom.sh $dev tape 
		;;
	esac

	;;


   directory )  
	dev="/"
	run_cmd ${HERE_ADD}/svr4/svr4.sh $drive $dev $ROOTDIR 
	;;
esac

clean
exit 0
