#
# Copyright 1995 Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)svr4.sh	1.29	95/12/01 SMI"
#
#UI-cmd#

trap 'exit 0' 1 2 3 15

authorized || exit 0

########
# clean (No Arg)
#
#	function to perform house cleaning at the end of the procedure
#	or when bail out for some other reasons such as failure.
#
clean()
{
	cleanup ${PKGNAME_LST} $INSTALL_LST $SCROLL_FORM $SCROLL_Y_INPUT $SCROLL_N_INPUT $FORM_INPUT $Yes_No_View
	freevar DB_KEY1 DB_KEY2 DWIDTH PKGNAME_LST VIEW HERE NONABI_SCRIPTS SCROLL_Y_INPUT SCROLL_N_INPUT FORM_INPUT SPACE_AVAIL SPACE_REQ_ALL YN
	return 0
}

. ../pkgadd_lib.sh			# Pull in the make_form function


#############################################################################
# Initialize variables:
#
#	Arg1	= Drive type 
#	Arg2	= Device 
#	Arg3	= Package Root Directory (Where to install packages)

drive=${1}
device=${2}
Target=${3}
[ ! -d ${Target}/usr/options ] && mkdir -p ${Target}/usr/options

setvar -ui DB_KEY1	""
setvar -ui DB_KEY2	""
setvar -ui PKGNAME_LST ${UITMPPATH}/pkgname.${CMD}
setvar -ui DWIDTH	41
setvar -ui HERE		`pwd`	# Used by the validation routine to find pkgview.sh

INSTALL_LST=${UITMPPATH}/install_lst.${CMD}  # List of packages to choose from
SCROLL_FORM=${UITMPPATH}/scroll_form.${CMD}  # contains UI definitions for form
setvar -ui FORM_INPUT ${UITMPPATH}/form_input.${CMD} # contains scrolling form input

# File containing scrolling form input with the first field set to yes
setvar -ui SCROLL_Y_INPUT ${UITMPPATH}/scroll_y_input.${CMD}
# File containing scrolling form input with the first field set to no 
setvar -ui SCROLL_N_INPUT ${UITMPPATH}/scroll_n_input.${CMD}

LIST_PKG=lspkg

Yes_No_View=${UITMPPATH}/yes_no_view.${CMD}		# Will hold yes/no/view
UI message lookup %YesNoView > $Yes_No_View

# $YES and $NO are set by sysadmin.sh
setvar -ui VIEW `sed -n '3p' $Yes_No_View`
setvar -ui NONABI_SCRIPTS TRUE				# See comment later about NONABI_SCRIPTS
setvar -ui SPACE_AVAIL
setvar -ui SPACE_REQ_ALL
setvar -ui YN N

########
# init_list: 
#
#	Initialize pick lists.
#

echo "../" > ${PKGNAME_LST}
if [ "$drive" = "directory" -o "$drive" = "cdrom" ]
then
	ls -1pL $device | cut -c1-${DWIDTH} >> ${PKGNAME_LST} 
else
	ls -1pL / | cut -c1-${DWIDTH} >> ${PKGNAME_LST} 
fi


########
# main(Arg1, Arg2. Arg3) 
#
#	Non-interactive SVR4 style package installation.
#

UI file ${CMDdotUI} ${DWIDTH} ${PKGNAME_LST}
case ${drive} in
   cdrom|directory)

	# Get Package Name and Directory that contains it.
	UI batch << EOF
		infomsg %dir_done_esc_m
		window select pkgdir
		form edit pkgdir "$device"
		picklist hide pkgname_pick
		form hide pkgdir
		window hide pkgdir
EOF
	eval `UI form get pkgdir`
	[ "$button" = "acancel" ] && clean && exit 0
	Src=${dir}
	setvar -ui PKG_DIR $Src      # The validation routine needs this

	UI infomsg %PackageI

	# OK, find out what packages are in the directory and build the
	# scrolling form based on what we find. Also, record the number
	# of packages at the same time so we know what length to make the
	# scrolling form. We will build two files containing input for the
	# scrolling form.  The first one will have the first field set to
	# yes and the second file will have the first field set to no
	$LIST_PKG -l "$YES" $Src > $SCROLL_Y_INPUT &
	NUM_PACKAGES=`$LIST_PKG -l "$NO" $Src | tee $SCROLL_N_INPUT | wc -l`
	cp $SCROLL_N_INPUT $FORM_INPUT
	make_form ${FORM_INPUT} ${NUM_PACKAGES} ${SCROLL_FORM}
	UI file ${SCROLL_FORM} ${Yes_No_View}

	# Set the space required by selected packages and space available on
	# the file system which user has chosen to install new packages in.
	KBYTES=`df -b $Target | awk '{ if ( NR == 2 ) print $2 }'` # free kbytes
	setvar -uiset SPACE_AVAIL `echo "scale=2;$KBYTES/1024" | bc`	# This is in MegaBytes
	SPACE_REQ=0

	# Get the total space required for all packages, in case user changes
	# defaults to yes.
	ALL_PKGNAMES=`cut -d\" -f4 $FORM_INPUT`
	TOTAL_SIZE=`$LIST_PKG -s $PKG_DIR $ALL_PKGNAMES` 
	setvar -uiset SPACE_REQ_ALL `echo "scale=2; $TOTAL_SIZE/1048576" | bc`

	UI batch <<-EOF
		clrinfo
		infomsg %form_use_i
		window select packages
		form edit packages ${SPACE_REQ} ${SPACE_AVAIL}
		form hide packages
		window hide packages
EOF

	eval `UI form get packages`
	UI batch <<-EOF
		variable free PKG_DIR
		window free packages
		form free packages
EOF
	[ "$button" = "cancel" ] && clean && exit 0 

	# PkgInst will hold a list of the package(s) we should install
	PkgInst=`awk '
	{
		if ($1 == yes)
			printf("%s ", $2)
	}' yes="\"$YES\"" $FORM_INPUT`

	PkgInst=`eval echo $PkgInst` 			# Drop the quotes

	####################################################################
	# By using the -a option for pkgadd and supplying an "admin"
	# file, we can run non-interactively. Save default admin file.
	####################################################################

	Log=/var/tmp/log.`echo $PkgInst | cut -d " " -f1`	# Use the first package as the extension
	> $Log
	AdminDir=$Target/var/sadm/install/admin
	AdminFile=$AdminDir/admin
	AdminSaved=$AdminDir/admin.orig
	[ ! -d ${AdminDir} ] && mkdir -p ${AdminDir}
	[ -f "${AdminFile}" ] && /usr/bin/cp $AdminFile $AdminSaved
	UI message lookup %admin_template > $AdminFile

	# Setting NONABI_SCRIPTS tells pkgadd/pkginstall to permit keyboard interaction
	# during the installation. Therefore we must run pkgadd in a window where
	# it is connected to a pseudo-terminal. It would appear that setting NONABI_SCRIPTS 
	# for ABI compliant scripts has no implications. We need to be a little creative to
	# get the return value from pkgadd as it is wrapped up in a 'tee' pipeline. The return
	# value, from UI, would be the tee but, we want the pkgadd. We get this by running the
	# whole lot in a subshell, with a echo $?, which is piped to to the log file. We then
	# pick $? out of the log file after it has finished.

	ABORTED=0
	FAILURES=0
	# Now add the package(s)
	for pkg in ${PkgInst}
	do
		UI batch<<-EOF
			variable set DB_KEY1 $pkg
			infomsg %pkgadd_wait_msg
			window select addpkg_dialog
		EOF
		UI wexec /usr/bin/sh -c "/usr/sbin/pkgadd -R $Target -n -a admin -d ${Src} ${pkg} 2>&1" LOG  $Log
		rv=$?
		UI window hide addpkg_dialog
		if [ "$rv" -ne 0 ]; then
			UI call errmsg $Log
			cp $Target/var/sadm/install/contents $Target/var/tmp/contents.${pkg}
			FAILURES=`expr $FAILURES + 1`
			# Ask if they want to continue
			UI variable set DB_KEY1 $pkg
			answer=`UI dbox question %InstFail %InstFailH NULL`
			[ $answer -ne 1 ] && ABORTED=1 && break
		else
			UI variable set DB_KEY1 $pkg
			UI variable set DB_KEY2 $Target
			UI infomsg %brief_v4_inst_ok
			sleep 2
		fi
	done
	
	setvar -ui LOG_FILE $Log
	if [ $ABORTED -eq 1 ] ; then
		setvar -ui NUM_FAILURES $FAILURES
		UI call notemsg %v4_inst_aborted
		freevar NUM_FAILURES
	else
		if [ $FAILURES -ne 0 ] ; then
			setvar -ui NUM_FAILURES $FAILURES
			UI call notemsg %v4_inst_failures
			freevar NUM_FAILURES
		else
			UI call notemsg %v4_inst_success
		fi
	fi
	freevar LOG_FILE

	# Restore default file if necessary.
	[ -f ${AdminSaved} ] && /usr/bin/mv $AdminSaved $AdminFile
	;;	

   tape?)
	UI batch <<EOF
		window select addpkg_dialog
		wexec /usr/sbin/pkgadd -R $Target -d ${device}
		window hide addpkg_dialog
EOF
	;;

   diskette?)
	UI batch <<EOF
		window select addpkg_dialog
		wexec /usr/sbin/pkgadd -R $Target -d ${device}
		infomsg %contkey
		waitkey
		window hide addpkg_dialog
EOF
	;;

esac

clean 
exit 0
