#
#
# Copyright 1995 Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)package.sh	1.19	95/10/19 SMI"
#
#UI-cmd#

trap 'exit 0' 1 2 3 15

########
# pkgclean (No Arg)
#
#	function to perform house cleaning at the end of the procedure
#	or when bail out for some other reasons such as failure.
#
pkgclean()
{
	CMD=${THISCMD}
	CURRWD=${HERE}
	cleanup ${PkgList}  ${DirList} ${PkgInfo} ${CustomPkgs} ${PkgType_file}
	freevar DirList DIRWIDTH PkgInfo CustomPkgs PkgType_file
}

########
# get_pkgs_list (1 Arg - Package ROOT directory)
#
#	Get both SVR4 and SVR3 package.
#
get_pkgs_list()
{
	PKGROOTDIR=${1}
	#
	# Get SVR4 and SVR3 Packages
	#
	pkginfo -R ${PKGROOTDIR} |
		tee $PkgInfo |
		sed 's/^[^ \	]*[ \	]*\([^ \  ]*\)[ \   ]*\(.*\)/\1: \2/g' | sort
	
	> $CustomPkgs
	#
	# Get SCO Packages
	#
	chkdepends -q %custom_pkgs && customlist -s | tee $CustomPkgs | sort

	#
	# Maintain file to later determine package type (especially for delete).
	#
	cat $PkgInfo | awk '{printf "%s %s\n", $2,$1}' > $PkgType_file
	if [ -s "$CustomPkgs" ] ; then
		cat $CustomPkgs | sed 's/\(.*\):.*/\1 custom/g' >> $PkgType_file
	fi
}


UI infomsg %package_m
####################
# initialize variables 
####################
setvar -ui PkgType_file  ${UITMPPATH}/PkgType.${CMD} 
setvar -ui PkgInfo  ${UITMPPATH}/PkgInfo.${CMD} 
setvar -ui CustomPkgs  ${UITMPPATH}/CustomPkgs.${CMD}
setvar -ui DirList  ${UITMPPATH}/dirlst.${CMD} 	# List of Directory Contents
setvar -ui DIRWIDTH 41 			# Max Width of picklist dir_item_pick

WIDTH=60				# Max Width of picklist listpkgs
PkgList=${UITMPPATH}/pkglst.${CMD}    	# List of Packages in Directory 
ROOTDIR=${PKGROOT:="/"}
export ROOTDIR
HERE=`pwd`
THISCMD=${CMD}

#############################################################################
# Initialize the list of installed packages ( "cut" to avoid overflowing the
# form horizontially).
#############################################################################

get_pkgs_list ${ROOTDIR} | cut -c1-${WIDTH} > ${PkgList} 
#############################################################################
# If no packages installed then fill in with " NO PACKAGES EXIST ON SYSTEM "
#  So user can still add.
# !!!!!!!!! Just until Sam allows empty picklists !!!!!!!!!!!!!!!!!!!!!
#############################################################################
if [ ! -s $PkgList ] ; then
	UI message lookup %NONE > ${PkgList}
fi

#############################################################################
# Initialize the content of the directory list.
#############################################################################
[ "$ROOTDIR" != "/" ] && echo "../" > ${DirList}
ls -1pL ${ROOTDIR} | cut -c1-${DIRWIDTH} >> ${DirList} 

#############################################################################
# display currently installed software
#############################################################################

UI file ${CMDdotUI} ${WIDTH} ${PkgList} ${DIRWIDTH} ${DirList}

while :
do
	######################################################################
	# list groups and allow user to add, modify, delete or view selection. 
	######################################################################

	UI batch << EOF
		clrinfo
		window select listpkgs
		form edit listpkgs "${ROOTDIR}"
EOF

	eval `UI form get listpkgs`
	Package=`echo $packages | cut -d":" -f1`

	case $button in

	view) 
		pkg_type=`egrep "^${Package} " $PkgType_file  | cut -d" " -f2`
		if [ "${pkg_type}" = "custom" ] ; then
			INSTALLDIR="/etc/perms"
		else
			INSTALLDIR="${ROOTDIR}"
		fi
		run_cmd ${HERE}/view/pkgview.sh "$Package" "$pkg_type" ${INSTALLDIR}
	 	;;

	add) 
		authorized -r || continue
		UI infomsg %pkgadd_m
		run_cmd ${HERE}/add/pkgadd.sh ${ROOTDIR}
	 	;;

	delete)
		authorized -r || continue
		pkg_type=`egrep "^${Package} " $PkgType_file  | cut -d" " -f2`
		run_cmd ${HERE}/delete/pkgdelete.sh "$Package" "$pkg_type" ${ROOTDIR}
		;;

	directory)
		UI infomsg %dir_done_esc_m
		ORG_ROOTDIR=${ROOTDIR}
		UI batch <<EOF
			window select main
			window select chroot
			infomsg %dir_done_esc_m
			form edit chroot "${ROOTDIR}"
			picklist hide dir_item_pick 
			form hide chroot
			window hide chroot
EOF
		eval `UI form get chroot`

		# If cancel was selected, we need to put the picklist back as
		# it was when we came in. Do this by simulating a change back
		# to the original root directory.
		[ "$button" = "rcancel" ] && button=change newroot=$ORG_ROOTDIR

		if [ "$button" = "change" ] ; then
			ROOTDIR=${newroot}
			[ "$ROOTDIR" != "/" ] && echo "../" > ${DirList}
			ls -1pL ${ROOTDIR} | cut -c1-${DIRWIDTH} >> ${DirList} 
			UI picklist reread dir_item_pick
		fi
		;;

	exit)
		break
		;;
	esac

	if [ "$button" != "view" ]
	then
		get_pkgs_list ${ROOTDIR} | cut -c1-${WIDTH} | sort > ${PkgList}
	fi
	UI picklist reread pkgs_pick
done

pkgclean
exit 0
