#!/sbin/sh
#
# Copyright 1995 Sun Microsystems, Inc.
# All rights reserved.
#
#ident	"@(#)sysadmin.sh	1.25	95/11/03 SMI"
#
#UI-cmd#

umask 077

#################
# Set variables #
#################

# Major

ADMROOT=/usr/snadm/sysadmin
ADMVAR=/var/sadm/system/admin/SUNWsyadm
SYSMANLIB=/usr/snadm/lib
PATH=${ADMROOT}/bin:${ADMROOT}/ui:${ADMROOT}/cli:/usr/bin:/usr/sbin
LD_LIBRARY_PATH=${SYSMANLIB}:/lib
UITMPPATH=/tmp/UI$$
export ADMROOT ADMVAR PATH LD_LIBRARY_PATH UITMPPATH

mkdir -p ${UITMPPATH}
cd ${ADMROOT}

# Minor

MENUDEF=${ADMROOT}/_menudef.ui
MAINMENUDIR=${ADMROOT}/mainmenu
ERRLOG=/tmp/sysadmin-err-log.$$
SYSLIBSH=${ADMROOT}/sysadmin_lib.sh
ERROR_FILE=${UITMPPATH}/errors
UI_WARNINGS_TO=/dev/null
YES_NO=${UITMPPATH}/yes_no	# File containing language independent "yes"
				# and "no" strings for use with ENUMs.  Will
				# also be used to initialize $YES and $NO
				# variables.
LIBUI="" LIBSH="" LIBVAL="" DEFAULTS="" CMDdotVAL="" CMDdotUI="" CMD=""
export MENUDEF MAINMENUDIR ERRLOG SYSLIBSH ERROR_FILE UI_WARNINGS_TO
export YES_NO
export LIBUI LIBSH LIBVAL DEFAULTS CMDdotVAL CMDdotUI CMD

####################################################################
# Figure what locale files to use and wether to switch font to 437 #
# Reset LC_*.  All locale aspects must be consistent becuase e.g.  #
# we cannot set the date in German if we are using English         #
# sysadmin text databases (comparisons go wrong).                  #
####################################################################

eval `getlocale` || exit 1
unset LC_CTYPE LC_COLLATE LC_TIME LC_NUMERIC LC_MONETARY LC_MESSAGES
if [ -d "/usr/lib/locale/${LOCALE}.${CODESET}" ] ; then
	LC_ALL=${LOCALE}.${CODESET}
else
	LC_ALL=${LOCALE}
fi
export TERM TEXTDOMAIN SWITCHFONT LC_ALL LOCALE CODESET
UI_MSGDB=${ADMROOT}/locale/${LOCALE}.${CODESET}/UIsys
SYSADMIN_MSGDB=${ADMROOT}/locale/${LOCALE}.${CODESET}/_sysadmin

if [ "$SWITCHFONT" = 1 ] ; then
	loadfont -d > ${UITMPPATH}/font.orig
	loadfont -c 437
fi

################################
# load commonly used functions #
################################

. ${ADMROOT}/sysadmin_lib.sh

trap 'sysadmin_abort' 1 2 3 15

######################################
# Start the UI server, check it's up #
######################################

eval `UI -h ${UI_MSGDB} init` || sysadmin_abort
export UIport
UI keyboard lock || sysadmin_abort
stty -onlcr < $UItty

UI color capable
if [ $? -ne 0 ] ; then
	UI color enable
fi

UI batch <<- EOF || sysadmin_abort
	#
	# Tell user we are initializing.
	# This key is stored in UIsys help DB.
	#
	infomsg %uiI
	#
	# Set framework variables.
	#
	define variable PATH		EXPORT ${PATH}
	define variable ADMROOT		EXPORT ${ADMROOT}
	define variable UITMPPATH	EXPORT ${UITMPPATH}
	define variable LD_LIBRARY_PATH	EXPORT ${LD_LIBRARY_PATH}
	define variable SYSLIBSH	EXPORT ${SYSLIBSH}
	define variable ERROR_FILE	EXPORT ${ERROR_FILE}
	define variable LIBUI		EXPORT ""
	define variable LIBSH		EXPORT ""
	define variable LIBVAL		EXPORT ""
	define variable DEFAULTS	EXPORT ""
	define variable CMDdotVAL	EXPORT ""
	define variable CMDdotUI	EXPORT ""
	define variable CMD		EXPORT "" 
	#
	# Select main help database.
	#
	msgdb select ${SYSADMIN_MSGDB}
	define hypertext "help" 4 4 16 70 "index"
	msgdb hypertext "help"
	#
	# Define commonly used functions.
	#
	file ${ADMROOT}/sysadmin_lib.ui
	#
	# Misc.
	#
	define variable Object
	define variable Obj2
	define variable VAL
	variable set MSG_EXPAND 1
	#
	define window main 2 0 19 78 SINGLE "%sysadmT"
	window select main
	#
	# Read/define menu hierarchy.
	#
	file ${MENUDEF}
	clrinfo
EOF

#
# Initialize file containing language-independent yes/no strings,
# and $YES and $NO shell/UI variables.
#

UI message lookup %YesNo > $YES_NO
setvar -ui YES `sed -n 1p $YES_NO`
setvar -ui NO `sed -n 2p $YES_NO`

#########################################
# standalone command  or run from menu? #
#########################################

MENU_OPTION=mainmenu
CMDFILE=${UITMPPATH}/cmdfile
if [ $# -ne 0 ] ; then
	# Get the first word of the first argument passed in ignoring "/"'s. 
	# Also translate all characters to lower case. 
	ARG1=`echo $1 | sed -e 's?/??' -e 's?^\([^ 	]*\).*?\1?' | tr "[A-Z]" "[a-z]"`
	shift 1
	args=$*
	case $ARG1 in
		add)	ARG1=harddisk  ;;	
		virtual) ARG1=vterm ;;
	esac

	if fgrep -i "M $ARG1 " ${MENUDEF} > $CMDFILE
	then
		# Need to handle dups
		num_found=`cat $CMDFILE | wc -l`
		[ "$num_found" -eq 1 ] && MENU_OPTION=`cut -d" " -f4 $CMDFILE`
	else
		MENU_OPTION=`fgrep "/${ARG1}.sh" ${MENUDEF} | cut -d" " -f4`
		[ -z "$MENU_OPTION" ] && MENU_OPTION=mainmenu
	fi
fi

############
# run menu #
############

if [ ! -f "${MENU_OPTION}" ] ; then
	ESC=FALSE
	UI infomsg %main_prompt
	action=`UI menu process ${MENU_OPTION} 0 ${ESC}`
	[ $? -ne 0 ] && sysadmin_abort

	while :
	do
    		CurMenu=`UI variable get UImname`
    		case "$action" in

		quit) 	#Exit main menu
	    		sysadmin_exit
	    		;;

		[0-9]|[0-9]*) #Exit standolalone menu
	    		break
	    		;;

		*)
	    		CMD="${action}"
	 		if [ -z "${CMD}" -o ! -f "${CMD}" ]
	   		then
				UI variable set Object \"${action}\"
				UI infomsg %nocmd
				sleep 1
	    		else
				run_cmd ${CMD} ${args}
	   		fi
	    		;;

	   	esac
		UI window bottom main
   		action=`UI menu resume ${MENU_OPTION}`
	done

#########################################
# standalone command  or run from menu? #
#########################################
else
	run_cmd ${MENU_OPTION} ${args}
fi

####################
# cleanup and exit #
####################

sysadmin_exit
exit 0
